package com.jpattern.gwt.client.history;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.jpattern.gwt.client.navigationevent.INavigationEvent;
import com.jpattern.gwt.client.navigationevent.INavigationEventWrapper;
import com.jpattern.gwt.client.navigationevent.NullNavigationEventWrapper;
import com.jpattern.gwt.client.presenter.IPresenter;
import com.jpattern.gwt.client.presenter.NullPresenter;
import com.jpattern.gwt.client.util.QueryString;

/**
 * 
 * @author Francesco Cina
 *
 * 27/lug/2011
 */
public class HistoryManager implements IHistoryManager {
	
	private static String TOKEN_SEPARATOR = "_";
	
	private final IHistoryManagerObserver historyManagerObserver;
	private final IHistoryEngine historyEngine;
	private INavigationEventWrapper rootNavigationEventWrapper = new NullNavigationEventWrapper();

	public HistoryManager(IHistoryEngine historyEngine, IHistoryManagerObserver historyManagerObserver) {
		this.historyManagerObserver = historyManagerObserver;
		this.historyEngine = historyEngine;
		historyEngine.init(this);
	}

	@Override
	public void onEvent(String eventName) {
		IEventToken eventToken = generateEventToken(eventName);
		String[] tokens = generateTokenList( eventToken.getToken() );
		Map<String, String> queryStringMap = QueryString.toMap(eventToken.getQueryString());
		launchEvents( eventName , tokens , queryStringMap);
	}

	@Override
	public void registerHistory(IPresenter parentPresenter, Map<String, String> queryStringValues, boolean registerEvent, INavigationEvent navigationEvent) {
		if (registerEvent) {
			List<String> presenterHierarchy = new ArrayList<String>();
			parentPresenter.hierarchy(presenterHierarchy);
			presenterHierarchy.add(navigationEvent.getName());
			
			String token = "";
			if (presenterHierarchy.size()>0 || queryStringValues.size()>0) {
				token = generateTokenString(presenterHierarchy);
				token = token + QueryString.toQueryString(queryStringValues);
				historyEngine.registerEvent(token);
			}
			historyManagerObserver.onRegisterHistory(token, presenterHierarchy);
		} else {
			navigationEvent.launch(parentPresenter, queryStringValues);
		}
	}

	public String generateTokenString(List<String> presenterHierarchy) {
		StringBuffer token = new StringBuffer();
		int size = presenterHierarchy.size();
		for (int i=0; i<size; i++) {
			token.append( presenterHierarchy.get(i) );
			if (i!=(size-1)) {
				token.append(TOKEN_SEPARATOR);
			}
		}
		return token.toString();
	}
	
	public String[] generateTokenList(String tokenString) {
		String[] result = tokenString.split(TOKEN_SEPARATOR);
		return result;
	}
	
	public void launchEvents(String eventName, String[] tokens , Map<String, String> queryStringMap) {
		List<IPresenter> createdPresenterList = new ArrayList<IPresenter>();
		rootNavigationEventWrapper.launchEvent(this, new NullPresenter(), createdPresenterList, eventName, tokens, 1, queryStringMap);
	}
	
	public void lauchEventCallBack(IPresenter parentPresenter, List<IPresenter> createdPresenterList, INavigationEvent navigationEvent, String eventName, String[] tokens , int tokenIndex, Map<String, String> queryStringMap) {
		if (tokenIndex<tokens.length) {
			String event = tokens[tokenIndex];
			parentPresenter = navigationEvent.launch(parentPresenter, queryStringMap, event);
			createdPresenterList.add(parentPresenter);
			parentPresenter.getNavigationEventData(event).getNavigationEventWrapper().launchEvent(this, parentPresenter, createdPresenterList, eventName, tokens , ++tokenIndex, queryStringMap);
		} else {
			parentPresenter = navigationEvent.launch(parentPresenter, queryStringMap);
			createdPresenterList.add(parentPresenter);
			historyManagerObserver.onEvent(eventName, createdPresenterList);
		}
	}
	
	@Override
	public void updateState() {
		historyEngine.updateState();		
	}
	
	public IEventToken generateEventToken(String eventName) {
		String queryString = "";
		String token = eventName;
		if (eventName.contains("?")) {
			token = eventName.substring(0, eventName.lastIndexOf("?"));
			queryString = eventName.substring(eventName.lastIndexOf("?"));
		}
		return new EventToken(token, queryString);
	}

	@Override
	public final void setRootNavigationEventWrapper(INavigationEventWrapper rootNavigationEventWrapper) {
		this.rootNavigationEventWrapper = rootNavigationEventWrapper;
	}

}
