package com.jpattern.gwt.client.navigationevent;

import java.util.Map;

import com.jpattern.gwt.client.presenter.IPresenter;

/**
 * 
 * An event to manage the web navigation through managed IPresenter creation 
 * 
 * @author Francesco Cina'
 *
 * 28/07/2011
 */
public interface INavigationEvent {

	/**
	 * Notify a new navigation event to the HistoryManager.
	 * A new presenter instance is created and associated to the parentPresenter.
	 * @param parentPresenter
	 * @param queryStringValues a map with key value pair to be added to the url generated by the navigation event
	 * @param whether register the event in the history manager or launch it directly
	 */
	void notifyNavigationEvent(IPresenter parentPresenter, Map<String, String> queryStringValues, boolean registerEvent);
	
	/**
	 * Launch a new navigation event using a specific parentPresenter.
	 * A new presenter instance is created and associated to the parentPresenter.
	 * @param parentPresenter
	 * @param queryStringValues a map with key value pair to be added to the url generated by the navigation event
	 */
	IPresenter launch(IPresenter parentPresenter, Map<String, String> queryStringValues);
	
	/**
	 * Launch a new navigation event using a specific parentPresenter.
	 * A new presenter instance is created and associated to the parentPresenter.
	 * @param parentPresenter
	 * @param queryStringValues a map with key value pair to be added to the url generated by the navigation event
	 * @param childInitialEventName an eventName to set as initial event for the created presenter
	 */
	IPresenter launch(IPresenter parentPresenter, Map<String, String> queryStringValues, String childInitialEventName);

	/**
	 * Return the name of the NavigationEvent
	 * @return
	 */
	String getName();
	
	/**
	 * Return whether the NavigationEvent uses always the same Presenter or creates a new one on every event launch.  
	 * @return
	 */
	boolean getSingletonPresenter();
	
	/**
	 * If set to true the NavigationEvent uses always the same presenter on every event launch.
	 * If set to false the NavigationEvent generates a new Presenter on every event.
	 * The default value is 'false'.
	 * @param singletonPresenter
	 */
	void setSingletonPresenter(boolean singletonPresenter);
	
}